;-------------------------------------------
;UnACEv2 example for MASM32v6 
;Version: v0.52
;Date created: 2001-10-09
;Time spent: 12 complete days
;Author: Florian Muecke (dev[AT]mueckeimnetz.de)
;-------------------------------------------
;Any comments, bugs, suggestions, problems ?
;Feel free to write back.
;-------------------------------------------


;
;=====---  Part 2: ACE.DLL callback declarations ---====
;

  ;---------------------------------------------------------------------------
  ;
  ; ACE.DLL makes use of four callback functions to exchange data
  ; with the application:
  ;
  ;   1) InfoCallbackProc   (pACEInfoCallbackProcStruc    Info)
  ;   2) ErrorCallbackProc  (pACEErrorCallbackProcStruc   Error)
  ;   3) RequestCallbackProc(pACERequestCallbackProcStruc Request)
  ;   4) StateCallbackProc  (pACEStateCallbackProcStruc   State)
  ;
  ; Meaning of different callback types:
  ;   Info    - lets the application know about actions that take some
  ;             time but are not essential
  ;             (Code is one of ACE_CALLBACK_INFO constants)
  ;   Error   - an error occured; if the reason for this error can
  ;             be solved by the application then the Dll can continue
  ;             the current operation, otherwise the operation has to
  ;             be canceled
  ;             (Code is one of ACE_CALLBACK_ERROR constants)
  ;   Request - the Dll needs some user input
  ;             for ex.: "Overwrite file? (yes/no/cancel)"
  ;             (Code is one of ACE_CALLBACK_REQUEST constants)
  ;   State   - Dll informs application about the progress of an operation
  ;             (Code is one of ACE_CALLBACK_STATE constants)
  ;
  ; The pointers to the callback functions has to be set by the application
  ; when calling ACEInitDll(). If the application does not install
  ; a callback function, is has set the corresponding pointer to NULL.
  ; If the ACE.DLL has to call the Error or Request callback function
  ; and they are not installed, the ACE.DLL will cancel the operation.
  ;
  ; The application has the possibility to cancel the current operation
  ; at each callback function call. So if the user clicks on a Cancel-button,
  ; the application should return ACE_CALLBACK_RETURN_CANCEL at the next
  ; callback function call.
  ;
  ; All callback function parameters are declared as UNIONs.
  ; The StructureType field contains he type of the STRUCTure which is used.
  ; When the application knows which type of STRUCTure it has to use,
  ; it will have to interpret the Code field to get to know the reason
  ; for the callback function call.
  ;
  ;---------------------------------------------------------------------------
  ;
  ; Contents:
  ;   Part 2.1: operation types
  ;   Part 2.2: callback function return codes
  ;   Part 2.3: callback STRUCTure types
  ;   Part 2.4: callback STRUCTures
  ;   Part 2.5: info callback function
  ;   Part 2.6: error callback function
  ;   Part 2.7: request callback function
  ;   Part 2.8: state callback function
  ;
  ;---------------------------------------------------------------------------


  ;
  ;=====================---  Part 2.1: operation codes  ---===================
  ;
  ; Passed to callback functions indicating the current operation.
  ;-------------------------------------------------------------------------

        ACE_CALLBACK_OPERATION_LIST			EQU 0
        ACE_CALLBACK_OPERATION_TEST			EQU 1
        ACE_CALLBACK_OPERATION_ANALYZE      EQU 2
        ACE_CALLBACK_OPERATION_EXTRACT      EQU 3
        ACE_CALLBACK_OPERATION_ADD          EQU 4
        ACE_CALLBACK_OPERATION_REPACK       EQU 5
        ACE_CALLBACK_OPERATION_DELETE       EQU 6
        ACE_CALLBACK_OPERATION_REPAIR       EQU 7    ; repair without
                                                     ; recovery record
        ACE_CALLBACK_OPERATION_SETCMT       EQU 8
        ACE_CALLBACK_OPERATION_ENCRYPT      EQU 9
        ACE_CALLBACK_OPERATION_KEEP         EQU 10   ; file is to be
                                                     ; taken along
                                                     ; without recompress
        ACE_CALLBACK_OPERATION_RECOVER      EQU 11   ; begin of
                                                     ; recovering archive
                                                     ; by recovery record
        ACE_CALLBACK_OPERATION_HEADSEARCH   EQU 12   ; begin of searching
                                                     ; for file headers
        ACE_CALLBACK_OPERATION_RECRECSEARCH EQU 13   ; begin of searching
                                                     ; for recovery record
        ACE_CALLBACK_OPERATION_ADDSFX       EQU 14
        ACE_CALLBACK_OPERATION_LOCK         EQU 15
        ACE_CALLBACK_OPERATION_ADDAV        EQU 16   ; not used in ACL
        ACE_CALLBACK_OPERATION_ADDRECOVREC  EQU 17
        ACE_CALLBACK_OPERATION_REGISTER     EQU 18   ; not used in ACL


  ;
  ;=============---  Part 2.2: callback function return codes  ---============
  ;
  ; One of these result codes has to be returned by the application-based
  ; callback functions.
  ;---------------------------------------------------------------------------

        ACE_CALLBACK_RETURN_OK      EQU 0    ; also "yes" at
                                             ; requests
        ACE_CALLBACK_RETURN_NO      EQU 1    ; no, do not/
                                             ; do not retry
        ACE_CALLBACK_RETURN_CANCEL	EQU 2    ; abort operation


  ;
  ;================---  Part 2.3: callback STRUCTure types  ---===============
  ;
  ; States of which type the passed STRUCTure is when a callback function
  ; is called. The type is written to the StructureType field.
  ;---------------------------------------------------------------------------

        ACE_CALLBACK_TYPE_GLOBAL    EQU 000000000h

                ; type of STRUCTure is
                ; tACECallbackGlobalStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; InfoCallbackProc    | ACE_CALLBACK_INFO_GENERALKEY      ; not used in ACL
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; ErrorCallbackProc   | ACE_CALLBACK_ERROR_MEMORY ; fr ManyFilesError und ExtractMem andere Codes verwenden!?
                ;                     | ACE_CALLBACK_ERROR_REGISTER       ; not used in ACL
                ;                     | ACE_CALLBACK_ERROR_READKEY        ; not used in ACL
                ;                     | ACE_CALLBACK_ERROR_WRITEKEY       ; not used in ACL
                ;                     | ACE_CALLBACK_ERROR_NOWINACEKEY    ; not used in ACL
                ;                     | ACE_CALLBACK_ERROR_NOACTIVEACEKEY ; not used in ACL
                ;                     | ACE_CALLBACK_ERROR_UNCSPACE       ; wird noch nicht verwendet!
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; RequestCallbackProc | ACE_CALLBACK_REQUEST_REGISTER     ; not used in ACL
                ;


        ACE_CALLBACK_TYPE_ARCHIVE		EQU 000000001h

                ; type of STRUCTure is
                ; tACECallbackArchiveStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; InfoCallbackProc    | ACE_CALLBACK_INFO_TMPARCCREATE
                ;                     | ACE_CALLBACK_INFO_TMPARCCREATEEND
                ;                     | ACE_CALLBACK_INFO_ADDRECREC
                ;                     | ACE_CALLBACK_INFO_ADDRECRECEND
                ;                     | ACE_CALLBACK_INFO_RECREC
                ;                     | ACE_CALLBACK_INFO_NORECREC
                ;                     | ACE_CALLBACK_INFO_RECOVERED
                ;                     | ACE_CALLBACK_INFO_NODAMAGE
                ;                     | ACE_CALLBACK_INFO_FNDMAINHEAD
                ;                     | ACE_CALLBACK_INFO_FILELISTCREATE
                ;                     | ACE_CALLBACK_INFO_FILELISTCREATEEND
                ;                     | ACE_CALLBACK_INFO_FILESORT
                ;                     | ACE_CALLBACK_INFO_FILESORTEND
                ;                     | ACE_CALLBACK_INFO_COPYEND
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; ErrorCallbackProc   | ACE_CALLBACK_ERROR_MODIFYVOLUME
                ;                     | ACE_CALLBACK_ERROR_MODIFYLOCKEDARCHIVE
                ;                     | ACE_CALLBACK_ERROR_AV                  ; not used in ACL
                ;                     | ACE_CALLBACK_ERROR_TOODAMAGED
                ;                     | ACE_CALLBACK_ERROR_ARCHIVEEXISTS
                ;                     | ACE_CALLBACK_ERROR_OPENREPAIRARCHIVE
                ;                     | ACE_CALLBACK_ERROR_OPENARCHIVEREAD
                ;                     | ACE_CALLBACK_ERROR_OPENARCHIVEWRITE
                ;                     | ACE_CALLBACK_ERROR_READARCHIVE
                ;                     | ACE_CALLBACK_ERROR_WRITEARCHIVE
                ;                     | ACE_CALLBACK_ERROR_ALREADYSFX
                ;                     | ACE_CALLBACK_ERROR_ADDSFXTOVOLUME
                ;                     | ACE_CALLBACK_ERROR_ARCHIVEBROKEN
                ;                     | ACE_CALLBACK_ERROR_ARCHIVESAVE
                ;                     | ACE_CALLBACK_ERROR_NOFILES
                ;                     | ACE_CALLBACK_ERROR_ISNOTANARCHIVE
                ;                     | ACE_CALLBACK_ERROR_TEMPDIRCREATE
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; RequestCallbackProc | ACE_CALLBACK_REQUEST_MARKASSOLID
                ;                     | ACE_CALLBACK_REQUEST_CHANGEVOLUME
                ;                     | ACE_CALLBACK_REQUEST_ARCHIVEEXISTS
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; StateCallbackProc   | ACE_CALLBACK_STATE_STARTARCHIVE
                ;


        ACE_CALLBACK_TYPE_ARCHIVEDFILE	EQU 000000002h
		
                ; type of STRUCTure is
                ; tACECallbackArchivedFileStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; InfoCallbackProc    | ACE_CALLBACK_INFO_TMPARCCREATE
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; ErrorCallbackProc   | ACE_CALLBACK_ERROR_CREATIONNAMEINUSE
                ;                     | ACE_CALLBACK_ERROR_HIGHERVERSION
                ;                     | ACE_CALLBACK_ERROR_ENCRYPTIONCRC
                ;                     | ACE_CALLBACK_ERROR_WRITE
                ;                     | ACE_CALLBACK_ERROR_READ
                ;                     | ACE_CALLBACK_ERROR_OPENREAD
                ;                     | ACE_CALLBACK_ERROR_OPENWRITE ;wird noch gar nich benutzt?? sollte aber - bei extract!
                ;                     | ACE_CALLBACK_ERROR_FILENAMETOOLONG
                ;                     | ACE_CALLBACK_ERROR_REPACKCRC
                ;                     | ACE_CALLBACK_ERROR_EXCLUDEPATH
                ;                     | ACE_CALLBACK_ERROR_METHOD
                ;                     | ACE_CALLBACK_ERROR_EXTRACTSPACE
                ;                     | ACE_CALLBACK_ERROR_CREATION
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; RequestCallbackProc | ACE_CALLBACK_REQUEST_OVERWRITE
                ;                     | ACE_CALLBACK_REQUEST_DELETEARCHIVEDSYSFILE
                ;                     | ACE_CALLBACK_REQUEST_ADDBROKENFILE
                ;                     | ACE_CALLBACK_REQUEST_PASSWORD
                ;                     | ACE_CALLBACK_REQUEST_OVERWRITESYSFILE
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; StateCallbackProc   | ACE_CALLBACK_STATE_STARTFILE
                ;                     | ACE_CALLBACK_STATE_ENDNOCRCCHECK
                ;


        ACE_CALLBACK_TYPE_REALFILE		EQU 000000003h

                ; type of STRUCTure is
                ; tACECallbackRealFileStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; InfoCallbackProc    | ACE_CALLBACK_INFO_FILELISTADD
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; ErrorCallbackProc   | ACE_CALLBACK_ERROR_MOVEDELETE
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; RequestCallbackProc | ACE_CALLBACK_REQUEST_MOVEDELETEREALSYSFILE
                ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
                ; StateCallbackProc   | ACE_CALLBACK_STATE_STARTFILE
                ;


        ACE_CALLBACK_TYPE_SPACE         EQU 000000004h

                ; type of STRUCTure is
                ; tACECallbackSpaceStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; ErrorCallbackProc   | ACE_CALLBACK_ERROR_TEMPDIRSPACE
                ;                     | ACE_CALLBACK_ERROR_ARCHIVESPACE
                ;


        ACE_CALLBACK_TYPE_SFXFILE       EQU 000000005h

                ; type of STRUCTure is
                ; tACECallbackSFXFileStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; ErrorCallbackProc   | ACE_CALLBACK_ERROR_READINGSFXFILE
                ;


        ACE_CALLBACK_TYPE_COPY          EQU 000000006h

                ; type of STRUCTure is
                ; tACECallbackCopyStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; InfoCallbackProc    | ACE_CALLBACK_INFO_COPY
                ;


        ACE_CALLBACK_TYPE_PROGRESS      EQU 000000007h
                
				; type of STRUCTure is
                ; tACECallbackProgressStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; StateCallbackProc   | ACE_CALLBACK_STATE_PROGRESS
                ;


        ACE_CALLBACK_TYPE_CRCCHECK      EQU 000000008h

                ; type of STRUCTure is
                ; tACECallbackCRCCheckStruc
                ;-------------------------------------------------------------
                ; callback function   | codes using this STRUCTure
                ; --- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
                ; StateCallbackProc   | ACE_CALLBACK_STATE_ENDCRCCHECK
                ;


  ;
  ;==============---  Part 2.4: different callback STRUCTures  ---============
  ;
  ; These are the declarations of the different STRUCTures used in the
  ; UNIONs passed by the callback functions.
  ;---------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; Only the Dll GlobalData is passed to the application.
      ;-----------------------------------------------------------------------

        tACECallbackGlobalStruc STRUCT 
        
          StructureType			ULONG 0 ; is ACE_CALLBACK_TYPE_GLOBAL
          Code					ULONG 0 ; see definition of
                                        ; ACE_CALLBACK_TYPE_GLOBAL
          Operation				ULONG 0 ; ACE_CALLBACK_OPERATION constant
          GlobalData			pACEGlobalDataStruc ?
										; see tACEGlobalDataStruc

        tACECallbackGlobalStruc ENDS 

        pACECallbackGlobalStruc	TYPEDEF PTR tACECallbackGlobalStruc


      ;-----------------------------------------------------------------------
      ; The Dll GlobalData and the ArchiveData are passed.
      ;-----------------------------------------------------------------------

        tACECallbackArchiveStruc STRUCT
        
          StructureType			ULONG 0 ; is ACE_CALLBACK_TYPE_ARCHIVE
          Code					ULONG 0 ; see definition of
                                        ; ACE_CALLBACK_TYPE_ARCHIVE
          Operation				ULONG 0 ; ACE_CALLBACK_OPERATION constant
          GlobalData			pACEGlobalDataStruc  ?
										; see tACEGlobalDataStruc
          ArchiveData			pACEArchiveDataStruc ?
										; see tACEArchiveDataStruc
        
		tACECallbackArchiveStruc ENDS

        pACECallbackArchiveStruc TYPEDEF PTR tACECallbackArchiveStruc


      ;-----------------------------------------------------------------------
      ; GlobalData, ArchiveData and FileData are passed.
      ;-----------------------------------------------------------------------

        tACECallbackArchivedFileStruc STRUCT
        
          StructureType			ULONG 0	; is ACE_CALLBACK_TYPE_ARCHIVEDFILE
          Code					ULONG 0 ; see definition of
                                        ; ACE_CALLBACK_TYPE_ARCHIVEDFILE
          Operation				ULONG 0 ; ACE_CALLBACK_OPERATION constant

		  GlobalData			pACEGlobalDataStruc ?
										; see tACEGlobalDataStruc
          ArchiveData			pACEArchiveDataStruc ?
										; see tACEArchiveDataStruc
          FileData				pACEFileDataStruc ?
										; see tACEFileDataStruc

		tACECallbackArchivedFileStruc ENDS

        pACECallbackArchivedFileStruc TYPEDEF PTR tACECallbackArchivedFileStruc


      ;-----------------------------------------------------------------------
      ; GlobalData, ArchiveData and a FileName are passed.
      ;-----------------------------------------------------------------------

        tACECallbackRealFileStruc STRUCT
        
          StructureType			ULONG 0 ; is ACE_CALLBACK_TYPE_REALFILE
          Code					ULONG 0 ; see definition of
                                        ; ACE_CALLBACK_TYPE_REALFILE
          Operation				ULONG 0	; ACE_CALLBACK_OPERATION constant

		  GlobalData			pACEGlobalDataStruc ?
									    ; see tACEGlobalDataStruc
          ArchiveData			pACEArchiveDataStruc ?
									    ; see tACEArchiveDataStruc
          FileName				LPSTR 0	; name of file

		tACECallbackRealFileStruc ENDS

		pACECallbackRealFileStruc TYPEDEF PTR tACECallbackRealFileStruc


      ;-----------------------------------------------------------------------
      ; GlobalData, ArchiveData, the path of temp directory and the
      ; bytes required in temp directory (archive size) are passed.
      ;-----------------------------------------------------------------------

        tACECallbackSpaceStruc STRUCT
        
          StructureType			ULONG 0	; is ACE_CALLBACK_TYPE_SPACE
          Code					ULONG 0 ; see definition of
                                        ; ACE_CALLBACK_TYPE_SPACE
          Operation				ULONG 0

          GlobalData			pACEGlobalDataStruc ?
										; see tACEGlobalDataStruc
          ArchiveData			pACEArchiveDataStruc ?
										; see tACEArchiveDataStruc
          Directory				LPSTR 0 ; path of directory
          ArchiveSize			QWORD 0 ; bytes required in temp dir

		tACECallbackSpaceStruc ENDS 

		pACECallbackSpaceStruc TYPEDEF PTR tACECallbackSpaceStruc


      ;-----------------------------------------------------------------------
      ; GlobalData, ArchiveData and SFXFileName are passed.
      ;-----------------------------------------------------------------------

        tACECallbackSFXFileStruc STRUCT
        
          StructureType			ULONG 0 ; is ACE_CALLBACK_TYPE_SFXFILE
          Code					ULONG 0 ; see definition of
                                        ; ACE_CALLBACK_TYPE_SFXFILE
          Operation				ULONG 0 ; ACE_CALLBACK_OPERATION constant

          GlobalData			pACEGlobalDataStruc ?
									    ; see tACEGlobalDataStruc
          ArchiveData			pACEArchiveDataStruc ?
									    ; see tACEArchiveDataStruc
          SFXFileName			LPSTR 0 ; name of SFX

        tACECallbackSFXFileStruc ENDS

        pACECallbackSFXFileStruc TYPEDEF PTR tACECallbackSFXFileStruc


      ;-----------------------------------------------------------------------
      ; GlobalData, ArchiveData and CopyData are passed.
      ;-----------------------------------------------------------------------

        tACECallbackCopyStruc STRUCT
        
          StructureType			ULONG 0 ; is ACE_CALLBACK_TYPE_COPY
          Code					ULONG 0 ; see definition of
                                        ; ACE_CALLBACK_TYPE_COPY
          Operation				ULONG 0 ; ACE_CALLBACK_OPERATION constant

          GlobalData			pACEGlobalDataStruc ?
										; see tACEGlobalDataStruc
          ArchiveData			pACEArchiveDataStruc ?
									    ; see tACEArchiveDataStruc
          CopyData				pACECopyInfoStruc ?
								        ; see tACECopyInfoStruc

        tACECallbackCopyStruc ENDS

        pACECallbackCopyStruc TYPEDEF PTR tACECallbackCopyStruc


      ;-----------------------------------------------------------------------
      ; GlobalData, ArchiveData, FileData and ProgressData are passed.
      ;-----------------------------------------------------------------------

        tACECallbackProgressStruc STRUCT
        
          StructureType			ULONG 0 ; is ACE_CALLBACK_TYPE_PROGRESS
          Code					ULONG 0 ; see definition of
                                        ; ACE_CALLBACK_TYPE_PROGRESS
          Operation             ULONG 0 ; ACE_CALLBACK_OPERATION constant

          GlobalData			pACEGlobalDataStruc ?
										; see tACEGlobalDataStruc
          ArchiveData			pACEArchiveDataStruc ?
										; see tACEArchiveDataStruc
          FileData				pACEFileDataStruc ?
										; see tACEFileDataStruc
          ProgressData			pACEProgressDataStruc ?
										; see tACEProgressDataStruc

		tACECallbackProgressStruc ENDS

		pACECallbackProgressStruc TYPEDEF PTR tACECallbackProgressStruc


      ;-----------------------------------------------------------------------
      ; GlobalData, ArchiveData, FileData and CRC-check result are passed.
      ;-----------------------------------------------------------------------

        tACECallbackCRCCheckStruc STRUCT
        
          StructureType			ULONG 0	; is ACE_CALLBACK_TYPE_CRCCHECK
          Code					ULONG 0 ; see definition of
										; ACE_CALLBACK_TYPE_CRCCHECK
          Operation             ULONG 0	; ACE_CALLBACK_OPERATION constant

          GlobalData			pACEGlobalDataStruc ?
										; see tACEGlobalDataStruc
          ArchiveData			pACEArchiveDataStruc ?
										; see tACEArchiveDataStruc
          FileData				pACEFileDataStruc ?
										; see tACEFileDataStruc
          CRCOk					BOOL 0  ; CRC-check result

        tACECallbackCRCCheckStruc ENDS

		pACECallbackCRCCheckStruc TYPEDEF PTR tACECallbackCRCCheckStruc


  ;
  ;=================---  Part 2.5: info callback function  ---================
  ;
  ; Declaration of ACEInfoCallbackProc() parameter and explanation of
  ; callback info codes.
  ;---------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; Union parameter used at ACEInfoCallbackProc().
      ;-----------------------------------------------------------------------

;;TYPEDEF
		tACEInfoCallbackProcStruc UNION 
        
          StructureType			ULONG 0	; indicates which of the fol-
                                        ; lowing STRUCTures is used
          Global_				tACECallbackGlobalStruc <>
          Archive_				tACECallbackArchiveStruc <>
          RealFile				tACECallbackRealFileStruc <>
          Copy					tACECallbackCopyStruc <>

        tACEInfoCallbackProcStruc ENDS 


      ;-----------------------------------------------------------------------
      ; These values are passed to the ACEInfoCallbackProc callback function
      ; to inform the application about actions (smaller parts of operations)
      ; which may take some time or other things that might be of interest.
      ;-----------------------------------------------------------------------

        ACE_CALLBACK_INFO_GENERALKEY    EQU 000000100h

                                ; key is a general one (no own AV; own key
                                ; is obtainable for a special price!?)
                                ; not used in ACL
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   ACERegister

        ACE_CALLBACK_INFO_TMPARCCREATE  EQU 000000110h

                                ; creating temporary archive for changes
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACERepair


        ACE_CALLBACK_INFO_TMPARCCREATEEND	EQU 000000111h

                                ; finished creating temporary archive
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACERepair


        ACE_CALLBACK_INFO_ADDRECREC     EQU 000000112h

                                ; adding recovery record
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord
                                ;   ACERepair


        ACE_CALLBACK_INFO_ADDRECRECEND  EQU 000000113h

                                ; finished adding recovery record
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord
                                ;   ACERepair


        ACE_CALLBACK_INFO_RECREC        EQU 000000114h

                                ; trying to recover files by recovery
                                ; record; end indicated by
                                ; ACE_CALLBACK_INFO_RECOVERED or
                                ; ACE_CALLBACK_INFO_NODAMAGE
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_INFO_NORECREC      EQU 000000115h

                                ; found no recovery record
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_INFO_RECOVERED     EQU 000000116h

                                ; archive has been fully recovered
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_INFO_NODAMAGE      EQU 000000117h

                                ; ACERepair detected by recovery record that
                                ; the archive is not damaged
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_INFO_FNDMAINHEAD   EQU 000000118h

                                ; found archive header
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_INFO_FILELISTCREATE	EQU 000000119h

                                ; creating a file list of specified files
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles


        ACE_CALLBACK_INFO_FILELISTCREATEEND	EQU 00000011ah

                                ; sent when creating the list of files
                                ; is finished
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles


        ACE_CALLBACK_INFO_FILESORT      EQU 00000011bh

                                ; sorting files (for solid compression)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd


        ACE_CALLBACK_INFO_FILESORTEND   EQU 00000011ch

                                ; sorting files (for solid compression)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd


        ACE_CALLBACK_INFO_COPYEND       EQU 00000011dh

                                ; copying a file finished
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACERepair


        ACE_CALLBACK_INFO_FILELISTADD   EQU 000000140h

                                ; called at creation of file list; the name
	                            ; of the file just added to file list is
                                ; passed in tACECallbackRealFileStruc
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_REALFILE
                                ; operations:
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles


        ACE_CALLBACK_INFO_COPY          EQU 000000150h

                                ; copying a file; file name, file size and
                                ; copied bytes are passed via
                                ; tACECallbackCopyStruc
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_COPY
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACERepair


  ;
  ;=================---  Part 2.6: error callback function  ---===============
  ;
  ; Declaration of ACEErrorCallbackProc() parameter and explanation of
  ; callback error codes.
  ;---------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; Union parameter used at ACEErrorCallbackProc().
      ;-----------------------------------------------------------------------

;;TYPEDEF
		tACEErrorCallbackProcStruc UNION 
        
          StructureType				ULONG 0	; indicates which of the fol-
                                            ; lowing STRUCTures is used
          Global_					tACECallbackGlobalStruc <>
          Archive_					tACECallbackArchiveStruc <>
          ArchivedFile				tACECallbackArchivedFileStruc <>
          RealFile					tACECallbackRealFileStruc <>
          Space						tACECallbackSpaceStruc <>
          SFXFile					tACECallbackSFXFileStruc <>

		tACEErrorCallbackProcStruc ENDS 


      ;-----------------------------------------------------------------------
      ; This STRUCTure is used by the ACEErrorCallback function to inform
      ; the application about errors. The Code field of the used STRUCTure
      ; contains an ACE_CALLBACK_ERROR value. At most problems modifications
      ; to the passed STRUCTure can be made to fix it. Other problems can not
      ; be solved and cause an operation abort immediately.
      ; ErrorCallbackProc() has to return either ACE_CALLBACK_RETURN_OK or
      ; ACE_CALLBACK_RETURN_CANCEL.
      ;-----------------------------------------------------------------------

        ACE_CALLBACK_ERROR_MEMORY		EQU 000000200h

                                ; not enough memory to perform operation
                                ; (dictionary too large?)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   all


        ACE_CALLBACK_ERROR_REGISTER     EQU 000000201h

                                ; registration key is invalid (or wrong
                                ; format?); not used in ACL
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   ACERegister


        ACE_CALLBACK_ERROR_READKEY      EQU 000000202h

                                ; key could not be read (does not exist or
                                ; is invalid); not used in ACL
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   ACEInitDll


        ACE_CALLBACK_ERROR_WRITEKEY     EQU 000000203h

                                ; could not write key; not used in ACL
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   ACERegister


        ACE_CALLBACK_ERROR_NOWINACEKEY  EQU 000000204h

                                ; key not valid for WinACE; not used in ACL
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   ACERegister


        ACE_CALLBACK_ERROR_NOACTIVEACEKEY	EQU 000000205h

                                ; key not valid for ActiveACE; not used in ACL
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   ACERegister


        ACE_CALLBACK_ERROR_UNCSPACE     EQU 000000206h

                                ; Win95_OSR1-bug: it is impossible to
                                ; get available space of network drives by
                                ; an UNC name; ACE will not stop but
                                ; assumes there are 4Gb free
                                ; --> the operation might fail if free
                                ; disk space is low
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   all


        ACE_CALLBACK_ERROR_MODIFYVOLUME EQU 000000220h

                                ; modification of volumes not possible
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord


        ACE_CALLBACK_ERROR_MODIFYLOCKEDARCHIVE	EQU 000000221h

                                ; modification of locked archive not possible
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord


        ACE_CALLBACK_ERROR_AV           EQU 000000222h

                                ; AV of archive is NOT ok or does not match
                                ; to the users AV (not used in ACL)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEReadArchiveData
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACERepair
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACEAddAV
                                ;   ACELock
                                ;   ACEAddRecoveryRecord


        ACE_CALLBACK_ERROR_TOODAMAGED   EQU 000000223h

                                ; can not repair by recovery record but
                                ; can continue with normal repair
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_ERROR_ARCHIVEEXISTS	EQU 000000224h

                                ; destination file name already used;
                                ; may occur if at
                                ; ACE_CALLBACK_ERROR_ARCHIVESPACE a
                                ; direcory is specified where a file
                                ; with the same name as the current archive
                                ; already exists
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd


        ACE_CALLBACK_ERROR_OPENREPAIRARCHIVE	EQU 000000225h

                                ; could not create archive for repairing
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_ERROR_OPENARCHIVEREAD	EQU 000000226h

                                ; could not open archive/volume for reading
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEReadArchiveData
                                ;   ACEList
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACERepair


        ACE_CALLBACK_ERROR_OPENARCHIVEWRITE	EQU 000000227h

                                ; could not open archive/volume for writing
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEDelete
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord
                                ;   ACERepair


        ACE_CALLBACK_ERROR_READARCHIVE  EQU 000000228h

                                ; error reading from archive
                                ; (source disk removed?)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEReadArchiveData
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACERepair
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACEAddAV
                                ;   ACELock
                                ;   ACEAddRecoveryRecord


        ACE_CALLBACK_ERROR_WRITEARCHIVE	EQU 000000229h

                                ; error writing to archive
                                ; (destination disk removed?)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEDelete
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord
                                ;   ACERepair


        ACE_CALLBACK_ERROR_ALREADYSFX   EQU 00000022ah

                                ; ca not make to SFX: is already SFX
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAddSFX


        ACE_CALLBACK_ERROR_ADDSFXTOVOLUME	EQU 00000022bh

                                ; adding SFX to volumes not possible
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAddSFX


        ACE_CALLBACK_ERROR_ARCHIVEBROKEN	EQU 00000022ch

                                ; archive is broken (damaged)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEReadArchiveData
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACERepair
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACEAddAV
                                ;   ACELock
                                ;   ACEAddRecoveryRecord

        ACE_CALLBACK_ERROR_ARCHIVESAVE  EQU 00000022dh

                                ; not enough space to save archive;
                                ; but normally
                                ; ACE_CALLBACK_ERROR_ARCHIVESPACE
                                ; should allow to change destination
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord
                                ;   ACERepair


        ACE_CALLBACK_ERROR_NOFILES      EQU 00000022eh

                                ; no files specified/could not find files
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles


        ACE_CALLBACK_ERROR_ISNOTANARCHIVE	EQU 00000022fh

                                ; specified archive file is not an
                                ; ACE archive
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEReadArchiveData
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACERepair
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACEAddAV
                                ;   ACELock
                                ;   ACEAddRecoveryRecord


        ACE_CALLBACK_ERROR_TEMPDIRCREATE	EQU 000000230h

                                ; could not create file in temp directory
                                ; (write protected or directory does
                                ;  not exist)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord
                                ;   ACERepair


        ACE_CALLBACK_ERROR_HIGHERVERSION	EQU 000000231h

                                ; this Dll version is not able to handle
                                ; the archive
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord


        ACE_CALLBACK_ERROR_CREATIONNAMEINUSE	EQU 000000240h

                                ; name used by directory
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEExtract


        ACE_CALLBACK_ERROR_ENCRYPTIONCRC	EQU 000000242h

                                ; encryption failed because of CRC-Error at
                                ; decompression
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEEncryptFiles


        ACE_CALLBACK_ERROR_READ         EQU 000000243h

                                ; error reading file to be added
                                ; (source disk removed?)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEAdd


        ACE_CALLBACK_ERROR_WRITE        EQU 000000244h

                                ; error at extraction
                                ; (destination disk removed?)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEExtract


        ACE_CALLBACK_ERROR_OPENREAD     EQU 000000245h

                                ; error opening file for reading
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEAdd


        ACE_CALLBACK_ERROR_OPENWRITE    EQU 000000246h

                                ; error opening file for writing
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEExtract


        ACE_CALLBACK_ERROR_FILENAMETOOLONG	EQU 000000247h

                                ; resulting file name too long
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEAdd


        ACE_CALLBACK_ERROR_REPACKCRC	EQU 000000248h

                                ; CRC-check error at recompression
                                ; (archive broken or wrong password)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEDelete
                                ;   ACEAdd


        ACE_CALLBACK_ERROR_EXCLUDEPATH	EQU 000000249h

                                ; could not exclude path of file names; two
                                ; or more files would have the same name
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEAdd


        ACE_CALLBACK_ERROR_METHOD		EQU 00000024ah

                                ; compression method not known to this
                                ; Dll version
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACEEncryptFiles


        ACE_CALLBACK_ERROR_EXTRACTSPACE	EQU 00000024bh

                                ; not enough space to extract file
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEExtract


        ACE_CALLBACK_ERROR_CREATION     EQU 00000024ch

                                ; creation failed (write-protection?)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEExtract


        ACE_CALLBACK_ERROR_OVERWRITEDELETE	EQU 00000024dh

                                ; could not overwrite because deletion of
                                ; file failed
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEExtract


        ACE_CALLBACK_ERROR_MOVEDELETE		EQU 000000260h

                                ; deletion of file or directory failed
                                ; (move operation)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_REALFILE
                                ; operations:
                                ;   ACEAdd

        ACE_CALLBACK_ERROR_TEMPDIRSPACE	EQU 000000270h

                                ; not enough space at current temp directory
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_SPACE
                                ; operations:
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACEAddAV


        ACE_CALLBACK_ERROR_ARCHIVESPACE	EQU 000000271h

                                ; not enough space to save archive
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_SPACE
                                ; operations:
                                ;   ACEDelete
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACELock
                                ;   ACEAddAV
                                ;   ACEAddRecoveryRecord
                                ;   ACERepair


        ACE_CALLBACK_ERROR_READINGSFXFILE	EQU 000000280h

                                ; error reading SFX file:
                                ; is no SFX file,
                                ; file does not exist or could not be opened
                                ; for reading
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_SFXFILE
                                ; operations:
                                ;   ACEAdd
                                ;   ACEAddSFX


  ;
  ;================---  Part 2.7: request callback function  ---==============
  ;
  ; Declaration of ACERequestCallbackProc() parameter and explanation of
  ; callback request codes.
  ;---------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; Union parameter used at ACERequestCallbackProc().
      ;-----------------------------------------------------------------------

;;TYPEDEF
		tACERequestCallbackProcStruc UNION 
        
          StructureType				ULONG 0	; indicates which of the fol-
                                            ; lowing STRUCTures is used
          Global					tACECallbackGlobalStruc <>
          Archive_					tACECallbackArchiveStruc <>
          ArchivedFile				tACECallbackArchivedFileStruc <>
          RealFile					tACECallbackRealFileStruc <>

        tACERequestCallbackProcStruc ENDS 


      ;-----------------------------------------------------------------------
      ; Question constants are passed to the RequestCallbackProc callback
      ; function to request further data.
      ; RequestCallbackProc may return ACE_CALLBACK_RETURN_OK,
      ; ACE_CALLBACK_RETURN_NO or ACE_CALLBACK_RETURN_CANCEL.
      ;-----------------------------------------------------------------------

        ACE_CALLBACK_REQUEST_REGISTER	EQU	000000300h

                                ; Global.UserAV has to be set
                                ; to continue registration process;
                                ; not used in ACL
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_GLOBAL
                                ; operations:
                                ;   ACERegister


        ACE_CALLBACK_REQUEST_MARKASSOLID	EQU	000000320h

                                ; ArchiveHeader damaged,
                                ; set solid flag for the new archive?
                                ; (in case of doubt return yes!)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_REQUEST_CHANGEVOLUME	EQU	000000321h
                                
								; Asks for permission to process next volume.
                                ; If operation is ACE_CALLBACK_OPERATION_ADD
                                ; then a new volume will be created.
                                ; The application may change the name
                                ; of the archive by modifying
                                ; ArchiveData->ArchiveName
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEDelete
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEList
                                ;   ACETest
                                ;   ACEExtract


        ACE_CALLBACK_REQUEST_ARCHIVEEXISTS	EQU	000000322h
                                
								; Asks whether to overwrite a file with
                                ; the same name as the archive.
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEDelete
                                ;   ACEAdd
                                ;   ACESetComments
                                ;   ACEEncryptFiles


        ACE_CALLBACK_REQUEST_OVERWRITE	EQU	000000340h

                                ; Overwrite existing file?
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEAdd
                                ;   ACEExtract


        ACE_CALLBACK_REQUEST_DELARCHIVEDSYSFILE	EQU	000000341h

                                ; Delete rdonly/hidden/system file
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEDelete


        ACE_CALLBACK_REQUEST_ADDBROKENFILE	EQU	000000342h

                                ; repair function found file with
                                ; broken header, add file?
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACERepair


        ACE_CALLBACK_REQUEST_PASSWORD	EQU	000000343h

                                ; password required; attention: may be
                                ; decryption _and_ encryption; but  passwords
                                ; can be different --> better changing
                                ; passwords at StateCallbackProc
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACEEncryptFiles


        ACE_CALLBACK_REQUEST_OVERWRITESYSFILE	EQU	000000344h

                                ; Overwrite rdonly/hidden/system file
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEAdd
                                ;   ACEExtract


        ACE_CALLBACK_REQUEST_MOVEDELREALSYSFILE	EQU	000000360h

                                ; Delete rdonly/hidden/system file
                                ; (move to archive operation)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_REALFILE
                                ; operations:
                                ;   ACEAdd


  ;
  ;=================---  Part 2.8: state callback function  ---===============
  ;
  ; Declaration of ACEStateCallbackProc() parameter and explanation of
  ; callback state codes.
  ;---------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; Union parameter used at ACEStateCallbackProc().
      ;-----------------------------------------------------------------------

;;TYPEDEF
        tACEStateCallbackProcStruc UNION
        
          StructureType			ULONG 0	; indicates which of the fol-
                                        ; lowing STRUCTures is used
          Archive_				tACECallbackArchiveStruc <>
          ArchivedFile			tACECallbackArchivedFileStruc <>
          RealFile				tACECallbackArchivedFileStruc <>
          Progress				tACECallbackProgressStruc <>
          CRCCheck				tACECallbackCRCCheckStruc <>

        tACEStateCallbackProcStruc ENDS


      ;-----------------------------------------------------------------------
      ; Calls to (*StateCallbackProc)() with ACE_CALLBACK_STATE values in the
      ; Code field are made to enable the application to show the progress of
      ; an operation.
      ;-----------------------------------------------------------------------

        ACE_CALLBACK_STATE_STARTARCHIVE	EQU 000000400h

                                ; procession of archive is about to begin
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVE
                                ; operations:
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACERepair
                                ;   ACESetComments
                                ;   ACEEncryptFiles
                                ;   ACEAddSFX
                                ;   ACEAddAV
                                ;   ACELock
                                ;   ACEAddRecoveryRecord


        ACE_CALLBACK_STATE_STARTFILE	EQU 000000410h

                                ; procession of file is about to begin
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACERepair
                                ;   ACESetComments
                                ;   ACEEncryptFiles


        ACE_CALLBACK_STATE_ENDNOCRCCHECK	EQU 000000410h

                                ; end of file procession
                                ; (no CRC chceck for this operation)
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_ARCHIVEDFILE
                                ; operations:
                                ;   ACEList
                                ;   ACEDelete
                                ;   ACEAdd
                                ;   ACERepair
                                ;   ACESetComments
                                ;   ACEEncryptFiles


        ACE_CALLBACK_STATE_PROGRESS		EQU 000000420h

                                ; informs about the progress of a file
                                ; operation
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_PROGRESS
                                ; operations:
                                ;   ACEDelete
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEAdd
                                ;   ACERepair
                                ;   ACEEncryptFiles


        ACE_CALLBACK_STATE_ENDCRCCHECK  EQU 000000430h

                                ; end of file procession, CRC-check
                                ; result is passed
                                ;---------------------------------------------
                                ; STRUCTure type:
                                ;   ACE_CALLBACK_TYPE_CRCCHECK
                                ; operations:
                                ;   ACETest
                                ;   ACEExtract
                                ;   ACEDelete
                                ;   ACEAdd
