;-------------------------------------------
;UnACEv2 example for MASM32v6 
;Version: v0.52
;Date created: 2001-10-09
;Time spent: 12 complete days
;Author: Florian Muecke (dev[AT]mueckeimnetz.de)
;-------------------------------------------
;Any comments, bugs, suggestions, problems ?
;Feel free to write back.
;-------------------------------------------

;
;=====---  Part 1: different STRUCTures ---====
;

  ;---------------------------------------------------------------------------
  ;
  ; Here in different STRUCTures used at callback functions and
  ; ACE.DLL functions are declared.
  ;
  ;---------------------------------------------------------------------------
  ;
  ; Contents:
  ;   Part 1.1: STRUCTures used in callback STRUCTures
  ;   Part 1.2: STRUCTures used in function STRUCTures
  ;
  ;---------------------------------------------------------------------------


  ;
  ;========---  Part 1.1: STRUCTures used in callback STRUCTures  ---=========
  ;

    ;  comment buffer STRUCTure  
    ; Used in tACEGlobalDataStruc. Application has to specify where the
    ; comment is or should be stored.
    ;-------------------------------------------------------------------------

		tACECommentStruc STRUCT 

		  Buf		LPSTR 0           ; area where the comments are stored
                                      ; by the application or by the Dll
                                      ; comments must be zero-terminated
          BufSize	ULONG 0           ; size of this area
          State		ULONG 0           ; state of comment - see ACE_COMMENT
                                      ; constants below
        tACECommentStruc ENDS

        pACECommentStruc TYPEDEF PTR tACECommentStruc


      ;-----------------------------------------------------------------------
      ; Defines for the tACECommentStruc.State field, which tells about
      ; the result of the last comment extraction.
      ;-----------------------------------------------------------------------

        ACE_COMMENT_OK			EQU	0    ; comment extraction went fine

        ACE_COMMENT_SMALLBUF	EQU	1    ; comment buffer too small to
                                         ; store the whole comment in

        ACE_COMMENT_NONE		EQU	255  ; No comment present


    ;  global data STRUCTure  
    ; This STRUCTure contains information for the Dll being interesting for
    ; nearly all functions. The Dll has to be initialized with this
    ; STRUCTure passed to tACEInitDll(). This STRUCTure is also passed
    ; by the callback functions.
    ;-------------------------------------------------------------------------

              ;---------------------------------------------------------------
              ; Forward declarations of callback function parameters
              ; used in this STRUCTure.
              ;---------------------------------------------------------------

				pACEInfoCallbackProcStruc TYPEDEF PTR tACEInfoCallbackProcStruc

				pACEErrorCallbackProcStruc TYPEDEF PTR tACEErrorCallbackProcStruc

                pACERequestCallbackProcStruc TYPEDEF PTR tACERequestCallbackProcStruc

                pACEStateCallbackProcStruc TYPEDEF PTR tACEStateCallbackProcStruc



		tACEGlobalDataStruc STRUCT 

		;------------------------ fields for ACE and UnACE -------------------

          Obj				DWORD 0 
		  
								; ---- reserved for application! ---- ("void  *Obj;")
                                ; thought to be used as a pointer to
                                ; an object; when a callback-function is
                                ; called, the object pointer can be used to
                                ; handle the callback in a specific way;
                                ; the pointer has to be initialized by
                                ; ACEInitDll()

          MaxArchiveTestBytes	ULONG 0

                                ; how many bytes of a file should be
                                ; looked upon at archive header search?

          MaxFileBufSize	ULONG 0

                                ; maximum buffer size for buffered
                                ; I/O operations

          Comment_			tACECommentStruc <>
								; used to exchange comment data
		                        ; between application and Dll
                                ; using callback functions

          DecryptPassword	LPSTR 0

                                ; the DecryptPassword specified at
                                ; ACEInitDll() is overwritten by the
                                ; DecryptPassword field of tACEAddStruc and
                                ; other function-specific STRUCTures;
                                ; but this field can be used to change the
                                ; password at callback function calls

          Reserved1			BYTE 64 DUP (0)
								
								; has to be filled with zeros

        ;--------------------------- fields for ACE only ---------------------

          EncryptPassword	LPSTR 0

                                ; things stated at the description of the
                                ; DecryptPassword field apply here as well


          TempDir			LPSTR 0 
		  
								; directory to save temporary archive

        ;-------- registration (ACE DLLs only, but not used at ACL) ----------

          KeyPath			LPSTR 0
						
								; necessary for reading and writing key file

          UserAV			LPSTR 0
								
								; Dll returns the AV string (if registered)
                                ; in this field

          IsGeneralKey		LPSTR 0
		  
								; DLL returns the key, if it is a general key

          OwnerWindow		HWND 0

								; specifies the applications window to be
                                ; parent of the registration reminder dialog

        ;--------------------------- fields for ACE only ---------------------

          CompressionLevel	ULONG 0

                                ; contains the currently used compression
                                ; level - may be changed during compression
                                ; operation

          Reserved2			BYTE 56 DUP (0)
								
								; has to be filled with zeros


        ;----------------------  callback functions (ACE and UnACE) ----------

          InfoCallbackProc	DWORD 0

								;;INT (__stdcall *InfoCallbackProc)
						        ;;	                (pACEInfoCallbackProcStruc Info);

                                ;  Informs application about details like the
                                ;  creation of a temporary archive.
                                ;  See ACE_CALLBACK_INFO definitions for
                                ;  more information.

          ErrorCallbackProc	DWORD 0
			
								;;INT (__stdcall *ErrorCallbackProc)
								;;               (pACEErrorCallbackProcStruc Error);

                                ;  Called to let the application know about
                                ;  errors. The Code field of Error will
                                ;  contain an ACE_CALLBACK_ERROR value.

          RequestCallbackProc	DWORD 0
		  
								;;INT (__stdcall *RequestCallbackProc)
								;;				 (pACERequestCallbackProcStruc Request);

                                ;  Called to let the application know about
                                ;  requests for data. Code will contain an
                                ;  ACE_CALLBACK_REQUEST value.

          StateCallbackProc	DWORD 0
		  
								;;INT (__stdcall *StateCallbackProc)
								;;				 (pACEStateCallbackProcStruc State);

                                ;  Called to let the application know about
                                ;  the progress of operations.


        ;-------------------------- different new fields ---------------------

		  Reserved3			BYTE 64 DUP (0)
				
								; has to be filled with zeros

		tACEGlobalDataStruc ENDS
		
		pACEGlobalDataStruc	TYPEDEF PTR tACEGlobalDataStruc


    ;  archive data STRUCTure  
    ; Informs the callback functions about the current archive, its volume
    ; number, the archive-flags (see ACE_FLAG constants), the creation host
    ; system (see ACE_HOST constants) and the AV if present in archive.
    ; Also used at ACEReadArchiveData().
    ;-------------------------------------------------------------------------

        tACEArchiveDataStruc STRUCT
         
		  ArchiveName		LPSTR 0
          VolumeNumber		ULONG 0
          Flags				ULONG 0			; see ACE_ARCFLAG defines below
          HostCreated		ULONG 0			; see ACE_HOST defines below
          TimeCreated		ULONG 0			; in MS-DOS format
          VersionCreated	ULONG 0
          VersionExtract	ULONG 0			; version needed to extract files
          AV				LPSTR 0			; not used in ACL
          Reserved			BYTE 64 DUP (0)	; filled with zeros

		 tACEArchiveDataStruc ENDS

		pACEArchiveDataStruc TYPEDEF PTR tACEArchiveDataStruc


      ;-----------------------------------------------------------------------
      ; Flag constants for tACEArchiveDataStruc.Flags field.
      ;-----------------------------------------------------------------------

        ACE_ARCFLAG_MAINCOMMENT	EQU	2
        ACE_ARCFLAG_SFX         EQU	512
        ACE_ARCFLAG_LIMITSFXJR  EQU	1024	; is an SFX archive
											; that supports 256k
                                            ; dictionary only
        ACE_ARCFLAG_MULTIVOLUME EQU	2048
        ACE_ARCFLAG_AV          EQU	4096	; not used in ACL
        ACE_ARCFLAG_RECOVERYREC EQU	8192
        ACE_ARCFLAG_LOCK        EQU	16384
        ACE_ARCFLAG_SOLID       EQU	32768


      ;-----------------------------------------------------------------------
      ; Host system used to create an archive. Used at
      ; tACEArchiveDataStruc.HostCreated field.
      ;-----------------------------------------------------------------------

        ACE_HOST_MSDOS			EQU	0     ; archive created by
                                          ; MSDOS ACE archiver
        ACE_HOST_OS2            EQU	1     ; created by OS2 ACE
        ACE_HOST_WIN32          EQU	2     ; created by Win32 ACE


    ;  file data STRUCTure  
    ; Contains information about an archived file.
    ;-------------------------------------------------------------------------

		tACEFileDataStruc STRUCT
        
          SourceFileName		LPSTR 0			; relative file name
          DestinationFileName	LPSTR 0			; absolute file name;
												; valid for add and extract only!
          Flags					ULONG 0			; see ACE_FILEFLAG defines below
          CRC32					ULONG 0
          Method				ULONG 0			; 0=stored, 1=LZ77, 2=V20Compression
          Dictionary			ULONG 0			; DictionarySize = 2^Dictionary
          CompressedSize		QWORD 0			;; DWORDLONG
          Size_					QWORD 0			;; DWORDLONG
          Time					ULONG 0
          Attributes			ULONG 0
          Reserved              BYTE 64 DUP (0)	; filled with zeros

		tACEFileDataStruc ENDS

		pACEFileDataStruc TYPEDEF PTR tACEFileDataStruc


      ;-----------------------------------------------------------------------
      ; Flag constants for the tACEFileData.Flags field.
      ;-----------------------------------------------------------------------

        ACE_FILEFLAG_FILECOMMENT	EQU	2     ; file has comment
        ACE_FILEFLAG_SPLITBEFORE    EQU	4096  ; continued from
                                              ; previous volume
        ACE_FILEFLAG_SPLITAFTER     EQU	8192  ; continued on
                                              ; next volume
        ACE_FILEFLAG_PASSWORD       EQU	16384 ; is encrypted
        ACE_FILEFLAG_SOLID          EQU	32768 ; uses data of previous
                                              ; files (solid mode)


    ;  copy info STRUCTure  
    ; Is passed to ACEInfoCallbackProc with ACE_CALLBACK_INFO_COPY as code.
    ; Informs application about the progress of copying either an archive to
    ; a temporary archive, or a temporary archive back to a normal archive.
    ;-------------------------------------------------------------------------

        tACECopyInfoStruc STRUCT
        
          SourceFileName        LPSTR 0         ; source file
          DestinationFileName   LPSTR 0         ; the file copying the source to
          CopiedBytes           QWORD 0         ; bytes already copied
          FileSize              QWORD 0         ; source file size
          Reserved              BYTE 64 DUP (0) ; filled with zeros

		tACECopyInfoStruc ENDS

		pACECopyInfoStruc TYPEDEF PTR tACECopyInfoStruc


    ;  operation progress STRUCTure  
    ; Used to state the progress of the current operation.
    ;-------------------------------------------------------------------------

		 tACEProgressDataStruc STRUCT
        
          Addr_                 LPSTR 0         ; address of uncompressed data block
          Size_                 ULONG 0         ; size of uncompressed data block
          TotalProcessedSize    QWORD 0         ; counted by Dll:
                                                ; total uncompressed bytes processed
          TotalCompressedSize   QWORD 0         ; total compressed bytes processed
          TotalSize             QWORD 0         ; total uncompressed bytes to process
                                                ; (sum of all original file sizes)
          FileProcessedSize     QWORD 0         ; uncompr. bytes of file processed
          FileCompressedSize    QWORD 0         ; compr. bytes of file processed
          FileSize              QWORD 0         ; uncompressed file size
        
		tACEProgressDataStruc ENDS
        
		pACEProgressDataStruc TYPEDEF PTR tACEProgressDataStruc


  ;
  ;========---  Part 1.2: STRUCTures used in function STRUCTures  ---=========
  ;

    ;  file list STRUCTure  
    ; This STRUCTure is used in the function specific STRUCTures.
    ; The application has to use this STRUCTure to indicate which files
    ; have to be processed by the DLL.
    ;-------------------------------------------------------------------------

		tACEFilesStruc STRUCT
        
          SourceDir             LPSTR 0         ; maybe a real or an archive directory
          FileList              LPSTR 0         ; pointer to list of files to process;
                                                ; zero-terminated; files have to be
                                                ; separated by carriage-return (0xd);
                                                ; FileList may/will be modified by the
                                                ; Dll; if nothing is specified, "*"
                                                ; will be used
                                                ; (attention at ACEDelete!!)
          ExcludeList           LPSTR 0         ; list of files to exclude from process
          FullMatch             BOOL 0          ; specifications must fully match
                                                ; (no files with the same name in
				                                ;  subdirs are processed)
          RecurseSubDirs        BOOL 0          ; recurse subdirectories
                                   			    ; (valid for add operations only)
          Reserved              BYTE 60 DUP (0) ; has to be filled with zeros
                                                ; for future: possibly in-/excluding
                                                ; file attributes and date/time range

		tACEFilesStruc ENDS 

		pACEFilesStruc TYPEDEF PTR tACEFilesStruc


    ;  V2.0 compression STRUCTure 
    ; Specifies whether to use v2.0 compression or not. If you use v2.0
    ; compression you can also specify which v2.0 compression techniques
    ; you want to use. (ACE only)
    ;-------------------------------------------------------------------------

		tACEV20CompressionStruc STRUCT
        
          DoUse                 BOOL 0          ; if DoUse=1 and all other fields are
          DoUseDelta            BOOL 0          ; zero, then all v2.0 compression
          DoUseExe              BOOL 0          ; techniques will be used
          DoUsePic              BOOL 0
          DoUseSound            BOOL 0
          Reserved              BYTE 64 DUP (0) ; has to be filled with zeros
        
		tACEV20CompressionStruc ENDS
        
		pACEV20CompressionStruc TYPEDEF PTR tACEV20CompressionStruc


    ;  compression parameter STRUCTure  
    ; Used in tACEAddStruc and tACEDeleteStruc. (ACE only)
    ;-------------------------------------------------------------------------

		tACECompressParamsStruc STRUCT
        
          Level                 ULONG 0         ; see ACE_LEVEL constants below
          Dictionary            ULONG 0         ; 15(32k)..22(4Mb)

          V20Compression        tACEV20CompressionStruc <>
                                    
                                                ; indicates if (or which) v2.0
                                                ; compression techniques shall be used

          TestAfter             BOOL 0          ; make a test for CRC check errors
                                                ; after compression
          Reserved              BYTE 64 DUP (0) ; has to be filled with zeros
        
		tACECompressParamsStruc ENDS
        
		pACECompressParamsStruc TYPEDEF PTR tACECompressParamsStruc


      ;-----------------------------------------------------------------------
      ; Tells the Dll which compression level to use. (ACE only)
      ;-----------------------------------------------------------------------

        ACE_LEVEL_STORE         EQU 0 ; save file only; do not compress
        ACE_LEVEL_FASTEST       EQU 1 ; compress very fast
        ACE_LEVEL_FAST          EQU 2 ; compress fast
        ACE_LEVEL_NORMAL        EQU 3 ; good compromise between speed and
                                      ; compression rate
        ACE_LEVEL_GOOD          EQU 4 ; achieves good compression
        ACE_LEVEL_BEST          EQU 5 ; best compression; bit slow

