;-------------------------------------------
;UnACEv2 example for MASM32v6 
;Version: v0.52
;Date created: 2001-10-09
;Time spent: 12 complete days
;Author: Florian Muecke (dev[AT]mueckeimnetz.de)
;-------------------------------------------
;Any comments, bugs, suggestions, problems ?
;Feel free to write back.
;-------------------------------------------

;
;=====---  Part 3: Functions supported by UNAceV2.Dll ---====
;

  ;---------------------------------------------------------------------------
  ;
  ; UnAceV2.Dll supports the following functions:
  ;
  ;   ACEInitDll
  ;   ACEReadArchiveData
  ;   ACEList
  ;   ACETest
  ;   ACEExtract
  ;
  ; First of all - before any other function is called - the Dll has to be
  ; initialized by ACEInitDll(). Using this function the application has
  ; to set temporary directory, key file path, comment buffer and
  ; callback function pointers.
  ; (callback function pointers can be set to NULL for first-try-runs)
  ;
  ; See description of each function for details about their task, input and
  ; output. Return codes of those functions are listed in part 3.1.
  ;
  ;---------------------------------------------------------------------------
  ;
  ; Contents:
  ;   Part 3.1: ACE.DLL function return codes
  ;   Part 3.2: functions and parameter STRUCTures
  ;
  ;---------------------------------------------------------------------------


  ;
  ;=============---  Part 3.1: ACE.DLL function return codes  ---=============
  ;

      ;-----------------------------------------------------------------------
      ; These error codes are returned by the ACE.DLL-functions. The meanings
      ; of the codes are the same, as they are for the exit codes of ACE.EXE.
      ;-----------------------------------------------------------------------

        ACE_ERROR_NOERROR	EQU	0   ; no error; operation succesful
        ACE_ERROR_MEM       EQU	1   ; insufficient memory
        ACE_ERROR_FILES     EQU	2   ; no files specified
        ACE_ERROR_FOUND     EQU	3   ; specified archive not found
        ACE_ERROR_FULL      EQU	4   ; disk full
        ACE_ERROR_OPEN      EQU	5   ; could not open file
        ACE_ERROR_READ      EQU	6   ; read error
        ACE_ERROR_WRITE     EQU	7   ; write error
        ACE_ERROR_CLINE     EQU	8   ; invalid command line
        ACE_ERROR_CRC       EQU	9   ; CRC error
        ACE_ERROR_OTHER     EQU	10  ; other error
        ACE_ERROR_EXISTS    EQU	1   ; file already exists
        ACE_ERROR_USER      EQU	255 ; user break (application
                                    ; returned cancel code at
                                    ; callback function)


      ;-----------------------------------------------------------------------
      ; These error codes are returned by the ACE.DLL-functions. They are not
      ; used by ACE.EXE yet.
      ;-----------------------------------------------------------------------

        ACE_ERROR_PARAM     EQU	128 ; might be used later


  ;
  ;============---  Part 3.2: functions and parameter STRUCTures ---==========
  ;

    ;  ACEInitDll  
    ; Initializes ACE dynamic link library. Has to be called before any
    ; other function call. May be called more than one time.
    ;-------------------------------------------------------------------------
    ; Input :
    ; Output:
    ;-------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; ACEInitDll() parameter STRUCTure.
      ;-----------------------------------------------------------------------

        tACEInitDllStruc STRUCT
        
          GlobalData		tACEGlobalDataStruc <>
          
          Reserved			BYTE 64 DUP (0)
							
								; has to be filled with zeroes

        tACEInitDllStruc ENDS 


		pACEInitDllStruc TYPEDEF PTR tACEInitDllStruc


      ;-----------------------------------------------------------------------
      ; ACEInitDll() function declaration.
      ;-----------------------------------------------------------------------

        ACEInitDll	PROTO STDCALL	DllData:pACEInitDllStruc


    ;  ACEReadArchiveData  
    ; Tests a file whether it is an archive or not and reads out the archive
    ; data.
    ;-------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; ACEReadArchiveData() parameter STRUCTure.
      ;-----------------------------------------------------------------------

        tACEReadArchiveDataStruc STRUCT
        
          ArchiveData		pACEArchiveDataStruc ?
								  ; if this pointer is NULL, the
							      ; file passed to ACEReadArchiveData
                                  ; is no archive; otherwise it points
                                  ; to a tACEArchiveDataStruc STRUCTure
                                  ; that contains information about the
                                  ; archive
          Reserved			BYTE 64 DUP (0)
								  ; has to be filled with zeroes

		tACEReadArchiveDataStruc ENDS

        pACEReadArchiveDataStruc TYPEDEF PTR tACEReadArchiveDataStruc


      ;-----------------------------------------------------------------------
      ; ACEReadArchiveData() function declaration.
      ;-----------------------------------------------------------------------

        ACEReadArchiveData	PROTO STDCALL	ArchiveName:LPSTR, ArchiveData:pACEReadArchiveDataStruc


    ;  ACEList  
    ; Passes the specified files in the archive to StateCallbackProc().
    ;-------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; ACEList() parameter STRUCTure.
      ;-----------------------------------------------------------------------

        tACEListStruc STRUCT 
        
          Files				tACEFilesStruc <>
								; specifies files to be listed;
						        ; see tACEFilesStruc STRUCTure
          Reserved			BYTE 64 DUP (0)
					            ; has to be filled with zeroes

		tACEListStruc ENDS

		pACEListStruc TYPEDEF PTR tACEListStruc


      ;-----------------------------------------------------------------------
      ; ACEList() function declaration.
      ;-----------------------------------------------------------------------

        ACEList		PROTO STDCALL	ArchiveName:LPSTR, List:pACEListStruc


    ;  ACETest  
    ; Tests specified files in archive.
    ;-------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; ACETest() parameter STRUCTure.
      ;-----------------------------------------------------------------------

        tACETestStruc STRUCT

          Files				tACEFilesStruc <>
								; specifies files to test;
                                ; see tACEFilesStruc STRUCTure
          DecryptPassword	LPSTR 0
							    ; zero-terminated string,
                                ; case-sensitive (maxlen=56)
          Reserved			BYTE 64 DUP (0)
								; has to be filled with zeroes

        tACETestStruc ENDS

        pACETestStruc TYPEDEF PTR tACETestStruc


      ;-----------------------------------------------------------------------
      ; ACETest() function declaration.
      ;-----------------------------------------------------------------------

        ACETest		PROTO STDCALL	ArchiveName:LPSTR, Test_:pACETestStruc


    ;  ACEExtract  
    ; Extracts specified  files.
    ;-------------------------------------------------------------------------

      ;-----------------------------------------------------------------------
      ; ACEExtract() parameter STRUCTure.
      ;-----------------------------------------------------------------------

        tACEExtractStruc STRUCT
        
          Files				tACEFilesStruc <>
								; specifies files to extract;
                                ; see tACEFilesStruc STRUCTure
          DestinationDir	LPSTR 0
						        ; directory to extract files to
          ExcludePath		BOOL 0
								; extract files without path
          DecryptPassword	LPSTR 0
							    ; password for decryption (if files
                                ; are encrypted);
                                ; zero-terminated string,
                                ; case-sensitive (maxlen=56)
          Reserved			BYTE 64 DUP (0)
								; has to be filled with zeroes

        tACEExtractStruc ENDS

		pACEExtractStruc TYPEDEF PTR tACEExtractStruc


      ;-----------------------------------------------------------------------
      ; ACEExtract() function declaration.
      ;-----------------------------------------------------------------------

        ACEExtract		PROTO STDCALL	ArchiveName:LPSTR, Extract:pACEExtractStruc

